#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <unistd.h>

/*

reinhard@finalmedia.de
Di 25. Apr 21:18:51 CEST 2023

example:  ./onlyif x y path/

read from stdin. line by line
write to stdout
if - and only if - a searchterm in
line is found at defined position.

all valid searchterms are defined at
runtime by checking an existing
filename in given path (directory).

define exact start and stop position
of searchterm in current line as global
setting at program start.

filter out all lines where no file exists
matching the given search term.

- exact matching and position required
- given path MUST end with trailing slash "/"
  else program will exit with exit immediately.
- x globally defines starting position for searchterm in each line
- y golbally defines end position for searchterm in each line
- x muss be smaller than y
- x must be between 1 and 512
- y must be between x+1 and 512
- given path doesn't need to exist at program start.
  full path gets checked at runtime each fetched line
- path can not exceed 512 chars
- a searchterm/filename MAY not exceed 512 chars. larger
  searchterms will get cut off.
- a searchterm MAY not start with dot "." or slash "/"
  lines are ignored, if these kind of searchterm is found.
- full path (path + filename) MAY not exceed 1024 chars.
  if full path is larged. path gets cut off.
- be aware: a searchterm in line MAY contain an slash "/"
  so it is possible to build sub path structures for checks
  but a searchterm MAY NOT contain a dot "." at any position
  so you can NOT use something like "valid/../../../etc/passwd".
  any "." or "\" in searchterm will get replace with "/"

*/
int main(int argc,const char *const *argv) {

if (argc<4) { fprintf(stderr,"example: %s 1 6 path/\n",argv[0]); return 1; }

size_t from = atoi(argv[1]);
size_t to = atoi(argv[2]);

if (from>=to) { fprintf(stderr,"error: searchterm start position must be smaller than end position\n"); return 2; }
if (to>512 || from<1) { fprintf(stderr,"error: searchterm position exceeds limits\n"); return 3; }
if (strlen(argv[3])>512 || strlen(argv[3])<1) { fprintf(stderr,"error: path length exceeds limit\n"); return 4; }
if (argv[3][strlen(argv[3])-1]!='/') { fprintf(stderr,"error: third parameter needs trailing slash\n"); return 5; }

while(1) {
	char *line = NULL;
	char search[512];
	char searchpath[1024];
	size_t len=0;
	if (getline(&line, &len, stdin)!=-1) {
		search[0]='\0';
		searchpath[0]='\0';
		strncpy(search,line+from-1,512);
		search[strlen(search)-1]='\0';
		search[to]='\0';
		/* important security check */
		if (strlen(search)>0 && strlen(search)<512 && search[0]!='.' && search[0]!='/') {
			if ((strlen(argv[3])+strlen(search))<1024) {
				/* another very important security mechanism */
				for(size_t i=0;i<strlen(search);i++) { if (search[i]=='.' || search[i]=='\\') { search[i]='/'; }}
				strncat(searchpath,argv[3],512-1);
				strncat(searchpath,search,1024-1);
				if(getenv("DEBUG")) { fprintf(stderr,"DEBUG: searchterm:'%s' searchpath:'%s' from:%ld to:%ld\n",search,searchpath,from,to); };
				if(getenv("VERBOSE")) { fprintf(stderr,"VERBOSE: %s",line); };
				if (access(searchpath, F_OK)==0) {
					printf("%s",line);
					fflush(stdout);
				}
			}
		}
	free(line);
	} else { return 0; }
}
}
